; 232-485 Transciever

;
; define PIC device type
;
;	device	pic16f84

;
; define config fuses
;
;	config	CP=off,WDT=off,PWRT=off,OSC=hs

	__config _wdt_off & _hs_osc
	
;
; include PIC register definitions, and some macros
;
	include "../include/p16f84.inc"
	include "../include/picmac_gpasm.h"


;
;                        -----------
;                       |     U     |
;            SPI CLK  --|RA2     RA1|-- 
;            SPI DIN  --|RA3     RA0|-- 
;           SPI DOUT  --|RA4        |
;                       |           | 
;                       |           |
;                       |           |
;             232_CS  --|RB0     RB7|-- 
;             485_CS  --|RB1     RB6|-- 
;             232_IRQ --|RB2     RB5|-- 
;             485_IRQ --|RB3     RB4|-- 485 RE/DE
;                       |           |
;                        -----------


#include "../include/uart_3110.h"

;
; bit definitions for two LEDs connected to port A bits 0 and 1
; bit masks can be computed from bit numbers with the left shift operator.
;

; this ordering is good for box 1
#define SPI_DIN PORTA,3
#define SPI_DOUT PORTA,4
#define SPI_CLK PORTA,2
#define UART_3110_232_CS PORTB,0
#define UART_3110_485_CS PORTB,1
#define UART_3110_232_IRQ PORTB,2
#define UART_3110_485_IRQ PORTB,3
#define UART_3110_485_DIR PORTB,4
#define UART_3110_232_PORT PORTB
#define UART_3110_485_PORT PORTB
#define UART_3110_232_MASK 1
#define UART_3110_485_MASK 2

#define BAUD_RATE_3110 B38400
#define MY_UART_CONFIG WRITE_CONFIGURATION | FIFO_INT_ENABLED | B38400 | PARITY_ENABLED

select_232_uart macro
	movlw	UART_3110_232_PORT
	movwf	uart_3110_port
	movlw	UART_3110_232_MASK
	movwf	uart_3110_mask
	endm
	
select_485_uart	macro
	movlw	UART_3110_485_PORT
	movwf	uart_3110_port
	movlw	UART_3110_485_MASK
	movwf	uart_3110_mask
	endm

;
; define some register file variables
;

	SEGU data
	org	0x0c

	SEG code

;
; code start
;
	org	0
	goto start

	org 4
	clrf	TMR0
	bcf		INTCON,T0IF
	bcf		INTCON,GIE
	bcf		UART_3110_485_DIR
	retfie
	
	
start

	clrf	PORTA
	clrf	PORTB
	
	bsf		UART_3110_232_CS
	bsf		UART_3110_485_CS
	

	bsf	STATUS,RP0			;register page 1
	movlw	B'00010000'			;all others output
	movwf	TRISA				;other PORTA pins as inputs
	movlw	B'00001100'			;b7,b6 input
	movwf	TRISB

	movlw	B'11010011'		; select 1:16 timer prescale
	movwf	OPTION_REG

	bcf	STATUS,RP0			;register page 0

	clrf	INTCON

	bsf	INTCON,T0IE		; turn on timer overflow interrupt
	bcf	INTCON,INTE		; turn off RB0 edge interrupt
	bcf	INTCON,RBIE

; globally enable interrupts
	bsf	INTCON,GIE

	select_232_uart
	
	movlw	HIGH(MY_UART_CONFIG)
	movwf	uart_3110_parmhi
	movlw	LOW(MY_UART_CONFIG)
	movwf	uart_3110_parmlo
	call	UART_232_INIT_PARM
	
	select_485_uart
	
	movlw	HIGH(MY_UART_CONFIG)
	movwf	uart_3110_parmhi
	movlw	LOW(MY_UART_CONFIG)
	movwf	uart_3110_parmlo
	call	UART_232_INIT_PARM


;testloop
	select_232_uart
	movlw	'J'
	call	UART_232_TX
	movlw	'P'
	call	UART_232_TX
	movlw	'0'
	call	UART_232_TX
;	
;	movlw	0xff
;	call	delay
;
;	goto testloop
	
main_loop		; essentially spin on either IRQ... if one is asserted, process it
	btfss	UART_3110_232_IRQ
	goto	handle_232_irq
	

	btfss	UART_3110_485_IRQ
	goto	handle_485_irq

	
	goto	main_loop

; need to properly handle the parity bit in here... just copy it, but still

handle_232_irq
	; we need to read in a byte from the 232 uart, then write it out to the 485 uart
	; a room for future improvement would detect if the 485 was currently receiving
	; before sending, since this could result in a bus conflict
	
	select_232_uart
	call	UART_232_RX
	
	movf	spi_high,W
	andlw	1
	movwf	uart_3110_parity

	
	
	select_485_uart
	movf	spi_low,W
	call	UART_232_TX

	bsf		UART_3110_485_DIR

	clrf	TMR0			; reset the timer
	bcf		INTCON,T0IF
	bsf		INTCON,GIE		; enable interrupt, when timer period passes with
							; no transmissions, then bus is relinquished


;	select_232_uart
;	movlw	'2'
;	call	UART_232_TX

	; hopefully the last bit will have been transmitted while were checking the
	; 232 interface...  At faster baudrates this becomes less of a problem, as the
	; UART_TX procedure occupies more and more of a baud period


	goto	main_loop

handle_485_irq
	select_485_uart
	call	UART_232_RX
	
	select_232_uart
	movf	spi_low,W
	call	UART_232_TX

;	movlw	'4'
;	call	UART_232_TX
	
	goto	main_loop
	
#include "../include/uart_3110_multiple.asm"
#include "../include/delay.asm"

	end
