#ifndef UART_3110
#define UART_3110

#include "../include/spi.asm"

; Interface to MAXIM 3110 UART using my SPI interface
;
; the CS pin is assumed to be at the mem location uart_3110_port
; and a bit mask is at uart_3110_mask

#include "../include/uart_3110.h"

; The following are the configuration bits that can be set



#define UART_3110_CONFIG WRITE_CONFIGURATION | BAUD_RATE_3110

	SEGU data
	
uart_3110_port DS 1
uart_3110_mask DS 1
uart_3110_temp DS 1	
uart_3110_temp2 DS 1
uart_3110_parmhi DS 1
uart_3110_parmlo DS 1

uart_rx_buf DS 1
uart_3110_parity DS 1
	
	SEG code

;----------------
; UART_INIT
;
; Called to initialize the UART with 
; a default configuration
;

UART_232_INIT

	movlw	HIGH(UART_3110_CONFIG)
	movwf	uart_3110_parmhi
	movlw	LOW(UART_3110_CONFIG)
	movwf	uart_3110_parmlo
	call	UART_232_INIT_PARM
	
	return
	

;----------------
; UART_INIT_PARM
;
; Called to initialize the UART with a passed config register
; in uart_3110_temp and uart_3110_temp2
;
UART_232_INIT_PARM

	movf	uart_3110_port,W
	movwf	FSR

	movf	uart_3110_parmhi,W
	movwf	spi_high
	movf	uart_3110_parmlo,W
	movwf	spi_low
	
	comf	uart_3110_mask,W	; load an inverted (negative) mask into W
	andwf	IND0,F				; clear the masked bit
	call	SPI_READWRITE16
	movf	uart_3110_mask,W	; load the normal mask into W
	iorwf	IND0,F				; set the CS bit
	
	movlw	HIGH(READ_CONFIGURATION)
	movwf	spi_high
	clrf	spi_low

	comf	uart_3110_mask,W	; load an inverted (negative) mask into W
	andwf	IND0,F				; clear the masked bit
	call	SPI_READWRITE16
	movf	uart_3110_mask,W	; load the normal mask into W
	iorwf	IND0,F				; set the CS bit
	
	movf	spi_low,W
	subwf	uart_3110_parmlo,W
	btfss	STATUS,Z
	goto	UART_232_INIT_PARM
	
	movlw	B'00111111'
	andwf	spi_high,F
	movf	uart_3110_parmhi,W
	andlw	B'00111111'
	subwf	spi_high,W
	btfss	STATUS,Z
	goto	UART_232_INIT_PARM
	
	retlw 0


;----------------
; UART_RX
;
; receives a byte into spi_low
; if no bytes are available 0 is returned in W otherwise
; 1 is returned...
;
; 6(7)+SPI_READ16=6(7)+138=144(145)
;

UART_232_RX
	bcf		SPI_DIN

	movf	uart_3110_port,W
	movwf	FSR

	comf	uart_3110_mask,W
	andwf	IND0,F
	call	SPI_READ16
	movf	uart_3110_mask,W
	iorwf	IND0,F

	btfss	spi_high,7
	retlw	0
	retlw	1


;----------------
; UART_232_TX_WAIT
;
; wait until a byte can be sent
UART_232_TX_WAIT
	movf	uart_3110_port,W
	movwf	FSR
	
u2tw_loop
	movlw	HIGH(READ_CONFIGURATION)
	movwf	spi_high
	clrf	spi_low
	
	comf	uart_3110_mask,W
	andwf	IND0,F
	call	SPI_READWRITE16
	movf	uart_3110_mask,W
	iorwf	IND0,F
	
	btfss	spi_high,6
	goto	u2tw_loop
	return

	
;----------------
; UART_TX
;
; the byte to send is in W
; and the appropriate parity bit is in uart_3110_parity
;

UART_232_TX
	; first have to wait to make sure the uart has room
	; for our character....
	movwf	uart_3110_temp

;	call	UART_232_TX_WAIT
	
	movf	uart_3110_temp,W
	movwf	spi_low
	movf	uart_3110_parity,W
	iorlw	HIGH(WRITE_DATA)
	movwf	spi_high

	comf	uart_3110_mask,W
	andwf	IND0,F
	call	SPI_READWRITE16
	movf	uart_3110_mask,W
	iorwf	IND0,F

	retlw 0

;----------------
; UART_232_PARITY_CHECK
;
; calculates the parity of the byte in spi_low and compares it against
; the received parity bit in spi_high.  If the conditions for odd parity
; are met, W=0 on return. otherwise w=1
;

UART_232_PARITY_CHECK
	movf	spi_low,W
	movwf	uart_3110_temp
	swapf	uart_3110_temp,F
	xorwf	uart_3110_temp,W
	movwf	uart_3110_temp
	rrf		uart_3110_temp,F
	rrf		uart_3110_temp,F
	xorwf	uart_3110_temp,W
	movwf	uart_3110_temp
	rrf		uart_3110_temp,F
	xorwf	uart_3110_temp,F
	movlw	1
	andwf	uart_3110_temp,F
	andwf	spi_high,W
	subwf	uart_3110_temp,W
	btfss	STATUS,Z
	retlw	0
	retlw	1

#endif
